@@ -1,3 +1,17 @@
+#   Copyright (C) 2008-2011 10gen Inc.
+# 
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+# 
+#      http://www.apache.org/licenses/LICENSE-2.0
+# 
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
+
 mongo.create <- function(host="127.0.0.1", name="", username="", password="", db="admin", timeout=0L) {
     mongo <- .Call(".mongo.create")
     attr(mongo, "host") <- host
@@ -168,3 +182,16 @@ mongo.get.databases <- function(mongo)
 mongo.get.database.collections <- function(mongo, db)
     .Call(".mongo.get.database.collections", mongo, db)
 
+mongo.distinct <- function(mongo, ns, key) {
+    pos <- regexpr('\\.', ns)
+    if (pos == 0) {
+        print("mongo.distict: No '.' in namespace")
+        return(NULL)
+    }
+    db <- substr(ns, 1, pos-1)
+    collection <- substr(ns, pos+1, nchar(ns))
+    b <- mongo.command(mongo, db, list(distinct=collection, key=key))
+    if (!is.null(b))
+        b <- mongo.bson.value(b, "values")
+    b
+}
